import { from } from 'most';
import { async as subject } from 'most-subject';
import { PING, PONG } from './config';
import { port } from './port';
import { EventConstructor } from './event';
import { EffectConstructor } from './effect';
export function createDomain(store, domainName = '') {
  var redux = store;
  var config = redux.dispatch({
    type: PING
  }).payload;

  var addPlainHandler = (name, fn) => {
    config.plain.set(name, fn);
  };

  var state$ = from(redux).map(() => store.getState()).multicast();
  return DomainConstructor(domainName, redux.dispatch, store.getState, state$, new Map(), addPlainHandler);
}
export function rootDomain(domainName = '') {
  var addPlainHandler = (name, fn) => {
    config.plain.set(name, fn);
  };

  var redux;
  var reduxStore;
  var config = {
    plain: new Map()
  };

  function dispatch(data) {
    return redux.dispatch(data);
  }

  function getState() {
    return reduxStore.getState();
  }

  var state$ = subject();

  function register(store) {
    redux = reduxStore = store;
    var pong = redux.dispatch({
      type: PING
    });

    if (pong) {
      var plain = pong.payload.plain;
      var oldPlain = config.plain;

      for (var [key, val] of oldPlain.entries()) {
        if (!plain.has(key)) {
          plain.set(key, val);
        }
      }

      config.plain = plain;
    } //$off


    from(store).map(() => getState()).observe(state => state$.next(state));
  }

  var domain = DomainConstructor(domainName, dispatch, getState, state$, new Map(), addPlainHandler);
  domain.register = register;
  return domain;
}

function DomainConstructor(domainName, dispatch, getState, state$, events = new Map(), addPlainHandler = () => {}) {
  return {
    port(events$) {
      port(dispatch, state$, events$);
    },

    register(store) {
      console.warn(`Not implemented`);
    },

    effect(name) {
      return EffectConstructor(domainName, dispatch, getState, state$, events, name);
    },

    domain(name) {
      return DomainConstructor([domainName, name].join('/'), dispatch, getState, state$, events, addPlainHandler);
    },

    event(name) {
      return EventConstructor(domainName, dispatch, getState, state$, events, name);
    },

    typeConstant(name) {
      var action$ = subject();
      addPlainHandler(name, data => {
        action$.next(data);
      });
      var result = EventConstructor('', dispatch, getState, state$, events, name, action$);
      return result;
    }

  };
}
//# sourceMappingURL=domain.js.map