import { subscribe } from 'most';
import { async as subject } from 'most-subject';
import { nextPayloadID, nextEventID, toTag, counter } from './id';
import { EventConstructor } from './event';
import { safeDispatch, port } from './port';
import { basicCommon } from './event-common';
export function EffectConstructor(domainName, dispatch, getState, state$, events, name, action$ = subject()) {
  var handlers = new Set();
  var {
    eventID,
    nextSeq,
    getType
  } = basicCommon(domainName, name);
  handlers.add((payload, state) => action$.next(payload)); // handlers.add((payload, state) => state$.next(state))

  var using = never;

  function use(thunk) {
    using = thunk;
  }

  var create = payload => ({
    type: getType(),
    payload,
    meta: {
      index: nextPayloadID(),
      eventID,
      seq: nextSeq()
    }
  });

  var effect = params => {
    var run = () => {
      run = () => {};

      using(params).then(result => {
        var data = {
          params,
          result
        };
        dispatch(done(data));
        resolve(data);
      }, error => {
        var data = {
          params,
          error
        };
        dispatch(fail(data));
        reject(data);
      });
    };

    var runner = dispatchHook => {
      if (canDispatch) {
        var dispatcher = dispatchHook ? dispatchHook : dispatch;
        canDispatch = false;
        dispatcher(result);

        for (var handler of handlers) {
          handler(params, getState());
        } // action$.next(params)


        run();
      }
    };

    var resolve = val => {};

    var reject = val => {};

    var doneP = new Promise(rs => resolve = rs);
    var failP = new Promise(rs => reject = rs);
    var toSend = Promise.resolve(params);
    var result = create(params);
    var canDispatch = true;
    return Object.assign({}, result, {
      raw() {
        return result;
      },

      send(dispatchHook) {
        runner(dispatchHook);
        return toSend;
      },

      done() {
        runner();
        return doneP;
      },

      fail() {
        runner();
        return failP;
      },

      promise() {
        runner();
        return Promise.race([doneP, failP.then(data => Promise.reject(data))]);
      }

    });
  };

  var done = EventConstructor(domainName, dispatch, getState, state$, events, [name, 'done'].join(' '));
  var fail = EventConstructor(domainName, dispatch, getState, state$, events, [name, 'fail'].join(' '));

  function watch(fn) {
    handlers.add((params, state) => {
      var result = fn(params, state);
      safeDispatch(result, dispatch);
    });
  }

  effect.use = use;
  effect.getType = getType;
  effect.done = done;
  effect.fail = fail;
  effect.watch = watch;
  effect.epic = port(dispatch, state$, action$);

  effect.subscribe = subscriber => subscribe(subscriber, action$);

  return effect;
}

function never(props) {
  console.warn(`


  Running an effect without implementation

`, props);
  return new Promise(() => {});
}
//# sourceMappingURL=effect.js.map