// import type {Store} from 'redux'
import { subscribe } from 'most';
import { async as subject } from 'most-subject';
import { nextPayloadID } from './id';
import { port } from './port';
import { basicCommon } from './event-common';
export function EventConstructor(domainName, dispatch, getState, state$, events, name, action$ = subject()) {
  var {
    eventID,
    nextSeq,
    getType
  } = basicCommon(domainName, name);
  var handlers = new Set();

  function watch(fn) {
    handlers.add(fn);
  }

  handlers.add((payload, state) => action$.next(payload)); // handlers.add((payload, state) => state$.next(state))

  var create = payload => ({
    type: getType(),
    payload,
    meta: {
      index: nextPayloadID(),
      eventID,
      seq: nextSeq()
    }
  });

  function eventInstance(payload) {
    var toSend = Promise.resolve(payload);
    var result = create(payload);
    var canDispatch = true;
    return Object.assign({}, result, {
      raw() {
        return result;
      },

      send(dispatchHook) {
        if (canDispatch) {
          var dispatcher = dispatchHook ? dispatchHook : dispatch;
          canDispatch = false;
          dispatcher(result);

          for (var handler of handlers) {
            handler(payload, getState());
          } // action$.next(payload)

        }

        return toSend;
      }

    });
  }

  eventInstance.getType = getType; //$off

  eventInstance.toString = getType;
  eventInstance.watch = watch;
  eventInstance.epic = port(dispatch, state$, action$);

  eventInstance.subscribe = subscriber => subscribe(subscriber, action$);

  events.set(getType(), eventInstance);
  return eventInstance;
}
//# sourceMappingURL=event.js.map