"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createDomain = createDomain;
exports.rootDomain = rootDomain;

var _most = require("most");

var _mostSubject = require("most-subject");

var _config = require("./config");

var _port = require("./port");

var _event = require("./event");

var _effect = require("./effect");

function createDomain(store, domainName = '') {
  var redux = store;
  var config = redux.dispatch({
    type: _config.PING
  }).payload;

  var addPlainHandler = (name, fn) => {
    config.plain.set(name, fn);
  };

  var state$ = (0, _most.from)(redux).map(() => store.getState()).multicast();
  return DomainConstructor(domainName, redux.dispatch, store.getState, state$, new Map(), addPlainHandler);
}

function rootDomain(domainName = '') {
  var addPlainHandler = (name, fn) => {
    config.plain.set(name, fn);
  };

  var redux;
  var reduxStore;
  var config = {
    plain: new Map()
  };

  function dispatch(data) {
    return redux.dispatch(data);
  }

  function getState() {
    return reduxStore.getState();
  }

  var state$ = (0, _mostSubject.async)();

  function register(store) {
    redux = reduxStore = store;
    var pong = redux.dispatch({
      type: _config.PING
    });

    if (pong) {
      var plain = pong.payload.plain;
      var oldPlain = config.plain;

      for (var [key, val] of oldPlain.entries()) {
        if (!plain.has(key)) {
          plain.set(key, val);
        }
      }

      config.plain = plain;
    } //$off


    (0, _most.from)(store).map(() => getState()).observe(state => state$.next(state));
  }

  var domain = DomainConstructor(domainName, dispatch, getState, state$, new Map(), addPlainHandler);
  domain.register = register;
  return domain;
}

function DomainConstructor(domainName, dispatch, getState, state$, events = new Map(), addPlainHandler = () => {}) {
  return {
    port(events$) {
      (0, _port.port)(dispatch, state$, events$);
    },

    register(store) {
      console.warn(`Not implemented`);
    },

    effect(name) {
      return (0, _effect.EffectConstructor)(domainName, dispatch, getState, state$, events, name);
    },

    domain(name) {
      return DomainConstructor([domainName, name].join('/'), dispatch, getState, state$, events, addPlainHandler);
    },

    event(name) {
      return (0, _event.EventConstructor)(domainName, dispatch, getState, state$, events, name);
    },

    typeConstant(name) {
      var action$ = (0, _mostSubject.async)();
      addPlainHandler(name, data => {
        action$.next(data);
      });
      var result = (0, _event.EventConstructor)('', dispatch, getState, state$, events, name, action$);
      return result;
    }

  };
}
//# sourceMappingURL=domain.js.map