"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.EventConstructor = EventConstructor;

var _most = require("most");

var _mostSubject = require("most-subject");

var _id = require("./id");

var _port = require("./port");

var _eventCommon = require("./event-common");

// import type {Store} from 'redux'
function EventConstructor(domainName, dispatch, getState, state$, events, name, action$ = (0, _mostSubject.async)()) {
  var {
    eventID,
    nextSeq,
    getType
  } = (0, _eventCommon.basicCommon)(domainName, name);
  var handlers = new Set();

  function watch(fn) {
    handlers.add(fn);
  }

  handlers.add((payload, state) => action$.next(payload)); // handlers.add((payload, state) => state$.next(state))

  var create = payload => ({
    type: getType(),
    payload,
    meta: {
      index: (0, _id.nextPayloadID)(),
      eventID,
      seq: nextSeq()
    }
  });

  function eventInstance(payload) {
    var toSend = Promise.resolve(payload);
    var result = create(payload);
    var canDispatch = true;
    return Object.assign({}, result, {
      raw() {
        return result;
      },

      send(dispatchHook) {
        if (canDispatch) {
          var dispatcher = dispatchHook ? dispatchHook : dispatch;
          canDispatch = false;
          dispatcher(result);

          for (var handler of handlers) {
            handler(payload, getState());
          } // action$.next(payload)

        }

        return toSend;
      }

    });
  }

  eventInstance.getType = getType; //$off

  eventInstance.toString = getType;
  eventInstance.watch = watch;
  eventInstance.epic = (0, _port.port)(dispatch, state$, action$);

  eventInstance.subscribe = subscriber => (0, _most.subscribe)(subscriber, action$);

  events.set(getType(), eventInstance);
  return eventInstance;
}
//# sourceMappingURL=event.js.map