"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.joint = joint;

var _reducer = require("./reducer");

//@noflow
function joint(combine, ...reducers) {
  var values = reducers.map(reducer => reducer(undefined, {
    type: '@@effector/void'
  }));
  var defaultValues = [...values];
  var defaultState = combine(...values);
  var lastState = defaultState;
  var combined = (0, _reducer.createReducer)(defaultState, {}, combinedReducer);

  function combinedReducer(state, action) {
    var changed = new Set();

    for (var i = 0; i < reducers.length; i++) {
      var prevValue = values[i];
      var value = reducers[i](prevValue, action);

      if (value !== prevValue) {
        changed.add(i);
        values[i] = value === undefined ? defaultValues[i] : value;
      }
    }

    var notChanged = changed.size === 0;
    var rawResult;
    if (notChanged) rawResult = lastState;else {
      lastState = rawResult = combine(...values);
    }
    if (rawResult === undefined) lastState = rawResult = defaultState;
    var result = combined(rawResult, action);
    if (result === undefined) return lastState = defaultState;
    return result;
  }

  Object.assign(combinedReducer, combined);
  return combinedReducer;
}
//# sourceMappingURL=joint.js.map